@@ -25,6 +25,7 @@
 #include <argp.h>
 #include <argp-namefrob.h>
 #include <argp-fmtstream.h>
+#include <argp-version-etc.h>
 
 #include <signal.h>
 #if ! defined SIGCHLD && defined SIGCLD
@@ -213,21 +214,28 @@ subcommand_string (enum subcommand c)
     case UPDATE_SUBCOMMAND:
       return "-u";
 
-    default:
-      abort ();
+    case TEST_LABEL_SUBCOMMAND:
+      return "--test-label";
+    }
+  abort ();
+}
+
+static void
+tar_list_quoting_styles (struct obstack *stk, char const *prefix)
+{
+  int i;
+  size_t prefixlen = strlen (prefix);
+
+  for (i = 0; quoting_style_args[i]; i++)
+    {
+      obstack_grow (stk, prefix, prefixlen);
+      obstack_grow (stk, quoting_style_args[i],
+		    strlen (quoting_style_args[i]));
+      obstack_1grow (stk, '\n');
     }
 }
 
-void
-tar_list_quoting_styles (argp_fmtstream_t fs, char *prefix)
-{
-  int i;
-
-  for (i = 0; quoting_style_args[i]; i++)
-    argp_fmtstream_printf (fs, "%s%s\n", prefix, quoting_style_args[i]);
-}
-
-void
+static void
 tar_set_quoting_style (char *arg)
 {
   int i;
@@ -256,6 +264,7 @@ enum
   DELAY_DIRECTORY_RESTORE_OPTION,
   HARD_DEREFERENCE_OPTION,
   DELETE_OPTION,
+  EXCLUDE_BACKUPS_OPTION,
   EXCLUDE_CACHES_OPTION,
   EXCLUDE_CACHES_UNDER_OPTION,
   EXCLUDE_CACHES_ALL_OPTION,
@@ -265,14 +274,15 @@ enum
   EXCLUDE_TAG_ALL_OPTION,
   EXCLUDE_VCS_OPTION,
   FORCE_LOCAL_OPTION,
+  FULL_TIME_OPTION,
   GROUP_OPTION,
-  HANG_OPTION,
   IGNORE_CASE_OPTION,
   IGNORE_COMMAND_ERROR_OPTION,
   IGNORE_FAILED_READ_OPTION,
   INDEX_FILE_OPTION,
   KEEP_NEWER_FILES_OPTION,
   LEVEL_OPTION,
+  LZIP_OPTION,
   LZMA_OPTION,
   LZOP_OPTION,
   MODE_OPTION,
@@ -290,6 +300,7 @@ enum
   NO_RECURSION_OPTION,
   NO_SAME_OWNER_OPTION,
   NO_SAME_PERMISSIONS_OPTION,
+  NO_SEEK_OPTION,
   NO_UNQUOTE_OPTION,
   NO_WILDCARDS_MATCH_SLASH_OPTION,
   NO_WILDCARDS_OPTION,
@@ -325,11 +336,9 @@ enum
   TO_COMMAND_OPTION,
   TRANSFORM_OPTION,
   UNQUOTE_OPTION,
-  USAGE_OPTION,
   UTC_OPTION,
-  VERSION_OPTION,
   VOLNO_FILE_OPTION,
-  WARNING_OPTION, 
+  WARNING_OPTION,
   WILDCARDS_MATCH_SLASH_OPTION,
   WILDCARDS_OPTION
 };
@@ -365,7 +374,12 @@ The version control may be set with --backup or VERSION_CONTROL, values are:\n\n
    [q  alias for --occurrence=1 =/= this would better be used for quiet?]
 
    y  per-file gzip compression
-   Y  per-block gzip compression */
+   Y  per-block gzip compression.
+
+   Additionally, the 'n' letter is assigned for option --seek, which
+   is probably not needed and should be marked as deprecated, so that
+   -n may become available in the future.
+*/
 
 static struct argp_option options[] = {
 #define GRID 10
@@ -419,6 +433,8 @@ static struct argp_option options[] = {
       " NUMBER defaults to 1"), GRID+1 },
   {"seek", 'n', NULL, 0,
    N_("archive is seekable"), GRID+1 },
+  {"no-seek", NO_SEEK_OPTION, NULL, 0,
+   N_("archive is not seekable"), GRID+1 },
   {"no-check-device", NO_CHECK_DEVICE_OPTION, NULL, 0,
    N_("do not check device numbers when creating incremental archives"),
    GRID+1 },
@@ -601,25 +617,21 @@ static struct argp_option options[] = {
   {"no-auto-compress", NO_AUTO_COMPRESS_OPTION, 0, 0,
    N_("do not use archive suffix to determine the compression program"),
    GRID+1 },
-  {"bzip2", 'j', 0, 0,
-   N_("filter the archive through bzip2"), GRID+1 },
-  {"gzip", 'z', 0, 0,
-   N_("filter the archive through gzip"), GRID+1 },
-  {"gunzip", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
-  {"ungzip", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
-  {"compress", 'Z', 0, 0,
-   N_("filter the archive through compress"), GRID+1 },
-  {"uncompress", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
-  {"lzma", LZMA_OPTION, 0, 0,
-   N_("filter the archive through lzma"), GRID+1 },
-  {"lzop", LZOP_OPTION, 0, 0,
-   N_("filter the archive through lzop"), GRID+8 },
-  {"xz", 'J', 0, 0,
-   N_("filter the archive through xz"), GRID+8 },
   {"use-compress-program", 'I', N_("PROG"), 0,
    N_("filter through PROG (must accept -d)"), GRID+1 },
+  /* Note: docstrings for the options below are generated by tar_help_filter */
+  {"bzip2", 'j', 0, 0, NULL, GRID+1 },
+  {"gzip", 'z', 0, 0, NULL, GRID+1 },
+  {"gunzip", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
+  {"ungzip", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
+  {"compress", 'Z', 0, 0, NULL, GRID+1 },
+  {"uncompress", 0, 0, OPTION_ALIAS, NULL, GRID+1 },
+  {"lzip", LZIP_OPTION, 0, 0, NULL, GRID+1 },
+  {"lzma", LZMA_OPTION, 0, 0, NULL, GRID+1 },
+  {"lzop", LZOP_OPTION, 0, 0, NULL, GRID+1 },
+  {"xz", 'J', 0, 0, NULL, GRID+1 },
 #undef GRID
-  
+
 #define GRID 100
   {NULL, 0, NULL, 0,
    N_("Local file selection:"), GRID },
@@ -659,6 +671,8 @@ static struct argp_option options[] = {
    N_("exclude directories containing FILE"), GRID+1 },
   {"exclude-vcs", EXCLUDE_VCS_OPTION, NULL, 0,
    N_("exclude version control system directories"), GRID+1 },
+  {"exclude-backups", EXCLUDE_BACKUPS_OPTION, NULL, 0,
+   N_("exclude backup and lock files"), GRID+1 },
   {"no-recursion", NO_RECURSION_OPTION, 0, 0,
    N_("avoid descending automatically in directories"), GRID+1 },
   {"one-file-system", ONE_FILE_SYSTEM_OPTION, 0, 0,
@@ -739,7 +753,9 @@ static struct argp_option options[] = {
       "Allowed signals are: SIGHUP, SIGQUIT, SIGINT, SIGUSR1 and SIGUSR2; "
       "the names without SIG prefix are also accepted"), GRID+1 },
   {"utc", UTC_OPTION, 0, 0,
-   N_("print file modification dates in UTC"), GRID+1 },
+   N_("print file modification times in UTC"), GRID+1 },
+  {"full-time", FULL_TIME_OPTION, 0, 0,
+   N_("print file time to its full resolution"), GRID+1 },
   {"index-file", INDEX_FILE_OPTION, N_("FILE"), 0,
    N_("send verbose output to FILE"), GRID+1 },
   {"block-number", 'R', 0, 0,
@@ -777,14 +793,6 @@ static struct argp_option options[] = {
 
   {"restrict", RESTRICT_OPTION, 0, 0,
    N_("disable use of some potentially harmful options"), -1 },
-
-  {"help",  '?', 0, 0,  N_("give this help list"), -1},
-  {"usage", USAGE_OPTION, 0, 0,  N_("give a short usage message"), -1},
-  {"version", VERSION_OPTION, 0, 0,  N_("print program version"), -1},
-  /* FIXME -V (--label) conflicts with the default short option for
-     --version */
-  {"HANG",	  HANG_OPTION,    "SECS", OPTION_ARG_OPTIONAL | OPTION_HIDDEN,
-   N_("hang for SECS seconds (default 3600)"), 0},
 #undef GRID
 
   {0, 0, 0, 0, 0, 0}
@@ -844,87 +852,82 @@ struct tar_args        /* Variables used during option parsing */
   | (args)->matching_flags \
   | recursion_option)
 
-void
-exclude_vcs_files ()
+static char const * const vcs_file_table[] = {
+  /* CVS: */
+  "CVS",
+  ".cvsignore",
+  /* RCS: */
+  "RCS",
+  /* SCCS: */
+  "SCCS",
+  /* SVN: */
+  ".svn",
+  /* git: */
+  ".git",
+  ".gitignore",
+  /* Arch: */
+  ".arch-ids",
+  "{arch}",
+  "=RELEASE-ID",
+  "=meta-update",
+  "=update",
+  /* Bazaar */
+  ".bzr",
+  ".bzrignore",
+  ".bzrtags",
+  /* Mercurial */
+  ".hg",
+  ".hgignore",
+  ".hgtags",
+  /* darcs */
+  "_darcs",
+  NULL
+};
+
+static char const * const backup_file_table[] = {
+  ".#*",
+  "*~",
+  "#*#",
+  NULL
+};
+
+static void
+add_exclude_array (char const * const * fv)
 {
   int i;
-  static char *vcs_file[] = {
-    /* CVS: */
-    "CVS",
-    ".cvsignore",
-    /* RCS: */
-    "RCS",
-    /* SCCS: */
-    "SCCS",
-    /* SVN: */
-    ".svn",
-    /* git: */
-    ".git",
-    ".gitignore",
-    /* Arch: */
-    ".arch-ids",
-    "{arch}",
-    "=RELEASE-ID",
-    "=meta-update",
-    "=update",
-    /* Bazaar */
-    ".bzr",
-    ".bzrignore",
-    ".bzrtags",
-    /* Mercurial */
-    ".hg",
-    ".hgignore",
-    ".hgtags",
-    /* darcs */
-    "_darcs",
-    NULL
-  };
 
-  for (i = 0; vcs_file[i]; i++)
-    add_exclude (excluded, vcs_file[i], 0);
+  for (i = 0; fv[i]; i++)
+    add_exclude (excluded, fv[i], 0);
 }
 
 
+static char *
+format_default_settings (void)
+{
+  return xasprintf (
+	    "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s"
 #ifdef REMOTE_SHELL
-# define DECL_SHOW_DEFAULT_SETTINGS(stream, printer)                      \
-{                                                                         \
-  printer (stream,                                                        \
-	   "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s",   \
-	   archive_format_string (DEFAULT_ARCHIVE_FORMAT),                \
-	   DEFAULT_ARCHIVE, DEFAULT_BLOCKING,                             \
-	   quoting_style_args[DEFAULT_QUOTING_STYLE],                     \
-	   DEFAULT_RMT_COMMAND);                                          \
-  printer (stream, " --rsh-command=%s", REMOTE_SHELL);                    \
-  printer (stream, "\n");                                                 \
-}
-#else
-# define DECL_SHOW_DEFAULT_SETTINGS(stream, printer)                      \
-{                                                                         \
-  printer (stream,                                                        \
-	   "--format=%s -f%s -b%d --quoting-style=%s --rmt-command=%s",   \
-	   archive_format_string (DEFAULT_ARCHIVE_FORMAT),                \
-	   DEFAULT_ARCHIVE, DEFAULT_BLOCKING,                             \
-	   quoting_style_args[DEFAULT_QUOTING_STYLE],                     \
-	   DEFAULT_RMT_COMMAND);                                          \
-  printer (stream, "\n");                                                 \
-}
+	    " --rsh-command=%s"
 #endif
+	    ,
+	    archive_format_string (DEFAULT_ARCHIVE_FORMAT),
+	    DEFAULT_ARCHIVE, DEFAULT_BLOCKING,
+	    quoting_style_args[DEFAULT_QUOTING_STYLE],
+	    DEFAULT_RMT_COMMAND
+#ifdef REMOTE_SHELL
+	    , REMOTE_SHELL
+#endif
+	    );
+}
 
-static void
-show_default_settings (FILE *fp)
-     DECL_SHOW_DEFAULT_SETTINGS(fp, fprintf)
-
-static void
-show_default_settings_fs (argp_fmtstream_t fs)
-     DECL_SHOW_DEFAULT_SETTINGS(fs, argp_fmtstream_printf)
-
+
 static void
 set_subcommand_option (enum subcommand subcommand)
 {
   if (subcommand_option != UNKNOWN_SUBCOMMAND
       && subcommand_option != subcommand)
     USAGE_ERROR ((0, 0,
-		  _("You may not specify more than one `-Acdtrux' option")));
+		  _("You may not specify more than one `-Acdtrux' or `--test-label' option")));
 
   subcommand_option = subcommand;
 }
@@ -963,14 +966,14 @@ stat_on_signal (int signo)
 #endif
 }
 
-void
+static void
 set_stat_signal (const char *name)
 {
   static struct sigtab
   {
-    char *name;
+    char const *name;
     int signo;
-  } sigtab[] = {
+  } const sigtab[] = {
     { "SIGUSR1", SIGUSR1 },
     { "USR1", SIGUSR1 },
     { "SIGUSR2", SIGUSR2 },
@@ -982,7 +985,7 @@ set_stat_signal (const char *name)
     { "SIGQUIT", SIGQUIT },
     { "QUIT", SIGQUIT }
   };
-  struct sigtab *p;
+  struct sigtab const *p;
 
   for (p = sigtab; p < sigtab + sizeof (sigtab) / sizeof (sigtab[0]); p++)
     if (strcmp (p->name, name) == 0)
@@ -997,12 +1000,12 @@ set_stat_signal (const char *name)
 struct textual_date
 {
   struct textual_date *next;
-  struct timespec *ts;
+  struct timespec ts;
   const char *option;
-  const char *date;
+  char *date;
 };
 
-static void
+static int
 get_date_or_file (struct tar_args *args, const char *option,
 		  const char *str, struct timespec *ts)
 {
@@ -1025,17 +1028,19 @@ get_date_or_file (struct tar_args *args, const char *option,
 	  WARN ((0, 0, _("Substituting %s for unknown date format %s"),
 		 tartime (*ts, false), quote (str)));
 	  ts->tv_nsec = 0;
+	  return 1;
 	}
       else
 	{
 	  struct textual_date *p = xmalloc (sizeof (*p));
-	  p->ts = ts;
+	  p->ts = *ts;
 	  p->option = option;
-	  p->date = str;
+	  p->date = xstrdup (str);
 	  p->next = args->textual_date;
 	  args->textual_date = p;
 	}
     }
+  return 0;
 }
 
 static void
@@ -1045,17 +1050,20 @@ report_textual_dates (struct tar_args *args)
   for (p = args->textual_date; p; )
     {
       struct textual_date *next = p->next;
-      char const *treated_as = tartime (*p->ts, true);
-      if (strcmp (p->date, treated_as) != 0)
-	WARN ((0, 0, _("Option %s: Treating date `%s' as %s"),
-	       p->option, p->date, treated_as));
+      if (verbose_option)
+	{
+	  char const *treated_as = tartime (p->ts, true);
+	  if (strcmp (p->date, treated_as) != 0)
+	    WARN ((0, 0, _("Option %s: Treating date `%s' as %s"),
+		   p->option, p->date, treated_as));
+	}
+      free (p->date);
       free (p);
       p = next;
     }
 }
 
 
-static volatile int _argp_hang;
 
 /* Either NL or NUL, as decided by the --null option.  */
 static char filename_terminator;
@@ -1159,7 +1167,7 @@ update_argv (const char *filename, struct argp_state *state)
   bool is_stdin = false;
   enum read_file_list_state read_state;
   int term = filename_terminator;
-  
+
   if (!strcmp (filename, "-"))
     {
       is_stdin = true;
@@ -1244,27 +1252,117 @@ update_argv (const char *filename, struct argp_state *state)
 }
 
 
-static void
-tar_help (struct argp_state *state)
+static char *
+tar_help_filter (int key, const char *text, void *input)
 {
-  argp_fmtstream_t fs;
-  state->flags |= ARGP_NO_EXIT;
-  argp_state_help (state, state->out_stream,
-		   ARGP_HELP_STD_HELP & ~ARGP_HELP_BUG_ADDR);
-  /* FIXME: use struct uparams.rmargin (from argp-help.c) instead of 79 */
-  fs = argp_make_fmtstream (state->out_stream, 0, 79, 0);
+  struct obstack stk;
+  char *s;
 
-  argp_fmtstream_printf (fs, "\n%s\n\n",
-		       _("Valid arguments for --quoting-style options are:"));
-  tar_list_quoting_styles (fs, "  ");
+  switch (key)
+    {
+    default:
+      s = (char*) text;
+      break;
 
-  argp_fmtstream_puts (fs, _("\n*This* tar defaults to:\n"));
-  show_default_settings_fs (fs);
-  argp_fmtstream_putc (fs, '\n');
-  argp_fmtstream_printf (fs, _("Report bugs to %s.\n"),
-			 argp_program_bug_address);
-  argp_fmtstream_free (fs);
+    case 'j':
+      s = xasprintf (_("filter the archive through %s"), BZIP2_PROGRAM);
+      break;
+
+    case 'z':
+      s = xasprintf (_("filter the archive through %s"), GZIP_PROGRAM);
+      break;
+
+    case 'Z':
+      s = xasprintf (_("filter the archive through %s"), COMPRESS_PROGRAM);
+      break;
+
+    case LZIP_OPTION:
+      s = xasprintf (_("filter the archive through %s"), LZIP_PROGRAM);
+      break;
+
+    case LZMA_OPTION:
+      s = xasprintf (_("filter the archive through %s"), LZMA_PROGRAM);
+      break;
+
+    case 'J':
+      s = xasprintf (_("filter the archive through %s"), XZ_PROGRAM);
+      break;
+
+    case ARGP_KEY_HELP_EXTRA:
+      {
+	const char *tstr;
+
+	obstack_init (&stk);
+	tstr = _("Valid arguments for the --quoting-style option are:");
+	obstack_grow (&stk, tstr, strlen (tstr));
+	obstack_grow (&stk, "\n\n", 2);
+	tar_list_quoting_styles (&stk, "  ");
+	tstr = _("\n*This* tar defaults to:\n");
+	obstack_grow (&stk, tstr, strlen (tstr));
+	s = format_default_settings ();
+	obstack_grow (&stk, s, strlen (s));
+	obstack_1grow (&stk, '\n');
+	obstack_1grow (&stk, 0);
+	s = xstrdup (obstack_finish (&stk));
+	obstack_free (&stk, NULL);
+      }
+    }
+  return s;
 }
+
+static char *
+expand_pax_option (struct tar_args *targs, const char *arg)
+{
+  struct obstack stk;
+  char *res;
+
+  obstack_init (&stk);
+  while (*arg)
+    {
+      size_t seglen = strcspn (arg, ",");
+      char *p = memchr (arg, '=', seglen);
+      if (p)
+	{
+	  size_t len = p - arg + 1;
+	  obstack_grow (&stk, arg, len);
+	  len = seglen - len;
+	  for (++p; *p && isspace ((unsigned char) *p); p++)
+	    len--;
+	  if (*p == '{' && p[len-1] == '}')
+	    {
+	      struct timespec ts;
+	      char *tmp = xmalloc (len);
+	      memcpy (tmp, p + 1, len-2);
+	      tmp[len-2] = 0;
+	      if (get_date_or_file (targs, "--pax-option", tmp, &ts) == 0)
+		{
+		  char buf[UINTMAX_STRSIZE_BOUND], *s;
+		  s = umaxtostr (ts.tv_sec, buf);
+		  obstack_grow (&stk, s, strlen (s));
+		}
+	      else
+		obstack_grow (&stk, p, len);
+	      free (tmp);
+	    }
+	  else
+	    obstack_grow (&stk, p, len);
+	}
+      else
+	obstack_grow (&stk, arg, seglen);
+
+      arg += seglen;
+      if (*arg)
+	{
+	  obstack_1grow (&stk, *arg);
+	  arg++;
+	}
+    }
+  obstack_1grow (&stk, 0);
+  res = xstrdup (obstack_finish (&stk));
+  obstack_free (&stk, NULL);
+  return res;
+}
+
 
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
@@ -1290,7 +1388,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case NO_AUTO_COMPRESS_OPTION:
       args->compress_autodetect = false;
       break;
-      
+
     case 'b':
       {
 	uintmax_t u;
@@ -1343,6 +1441,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       multi_volume_option = true;
       break;
 
+    case FULL_TIME_OPTION:
+      full_time_option = true;
+      break;
+
     case 'g':
       listed_incremental_option = arg;
       after_date_option = true;
@@ -1364,7 +1466,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case HARD_DEREFERENCE_OPTION:
       hard_dereference_option = true;
       break;
-      
+
     case 'i':
       /* Ignore zero blocks (eofs).  This can't be the default,
 	 because Unix tar writes two blocks of zeros, then pads out
@@ -1374,13 +1476,13 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'j':
-      set_use_compress_program_option ("bzip2");
+      set_use_compress_program_option (BZIP2_PROGRAM);
       break;
 
     case 'J':
-      set_use_compress_program_option ("xz");
+      set_use_compress_program_option (XZ_PROGRAM);
       break;
-      
+
     case 'k':
       /* Don't replace existing files.  */
       old_files_option = KEEP_OLD_FILES;
@@ -1420,15 +1522,19 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	  USAGE_ERROR ((0, 0, _("Invalid incremental level value")));
       }
       break;
-      
+
+    case LZIP_OPTION:
+      set_use_compress_program_option (LZIP_PROGRAM);
+      break;
+
     case LZMA_OPTION:
-      set_use_compress_program_option ("lzma");
+      set_use_compress_program_option (LZMA_PROGRAM);
       break;
-      
+
     case LZOP_OPTION:
-      set_use_compress_program_option ("lzop");
+      set_use_compress_program_option (LZOP_PROGRAM);
       break;
-      
+
     case 'm':
       touch_option = true;
       break;
@@ -1446,7 +1552,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'n':
-      seekable_archive = true;
+      seek_option = 1;
+      break;
+
+    case NO_SEEK_OPTION:
+      seek_option = 0;
       break;
 
     case 'N':
@@ -1458,7 +1568,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	USAGE_ERROR ((0, 0, _("More than one threshold date")));
       get_date_or_file (args,
 			key == NEWER_MTIME_OPTION ? "--newer-mtime"
-			  : "--after-date", arg, &newer_mtime_option);
+			: "--after-date", arg, &newer_mtime_option);
       break;
 
     case 'o':
@@ -1524,8 +1634,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case TEST_LABEL_OPTION:
-      set_subcommand_option (LIST_SUBCOMMAND);
-      test_label_option = true;
+      set_subcommand_option (TEST_LABEL_SUBCOMMAND);
       break;
 
     case 'T':
@@ -1580,11 +1689,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'z':
-      set_use_compress_program_option ("gzip");
+      set_use_compress_program_option (GZIP_PROGRAM);
       break;
 
     case 'Z':
-      set_use_compress_program_option ("compress");
+      set_use_compress_program_option (COMPRESS_PROGRAM);
       break;
 
     case ANCHORED_OPTION:
@@ -1606,11 +1715,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case CHECK_DEVICE_OPTION:
       check_device_option = true;
       break;
-      
+
     case NO_CHECK_DEVICE_OPTION:
       check_device_option = false;
       break;
-      
+
     case CHECKPOINT_OPTION:
       if (arg)
 	{
@@ -1633,7 +1742,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case CHECKPOINT_ACTION_OPTION:
       checkpoint_compile_action (arg);
       break;
-      
+
     case BACKUP_OPTION:
       backup_option = true;
       if (arg)
@@ -1652,6 +1761,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       set_subcommand_option (DELETE_SUBCOMMAND);
       break;
 
+    case EXCLUDE_BACKUPS_OPTION:
+      add_exclude_array (backup_file_table);
+      break;
+
     case EXCLUDE_OPTION:
       add_exclude (excluded, arg, MAKE_EXCL_OPTIONS (args));
       break;
@@ -1684,9 +1797,9 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case EXCLUDE_VCS_OPTION:
-      exclude_vcs_files ();
+      add_exclude_array (vcs_file_table);
       break;
-      
+
     case FORCE_LOCAL_OPTION:
       force_local_option = true;
       break;
@@ -1725,7 +1838,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	    group_option = g;
 	  else
 	    FATAL_ERROR ((0, 0, "%s: %s", quotearg_colon (arg),
-			  _("%s: Invalid group")));
+			  _("Invalid group")));
 	}
       break;
 
@@ -1825,8 +1938,12 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case PAX_OPTION:
-      args->pax_option = true;
-      xheader_set_option (arg);
+      {
+	char *tmp = expand_pax_option (args, arg);
+	args->pax_option = true;
+	xheader_set_option (tmp);
+	free (tmp);
+      }
       break;
 
     case POSIX_OPTION:
@@ -1877,9 +1994,13 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case SHOW_DEFAULTS_OPTION:
-      show_default_settings (stdout);
-      close_stdout ();
-      exit (0);
+      {
+	char *s = format_default_settings ();
+	printf ("%s\n", s);
+	close_stdout ();
+	free (s);
+	exit (0);
+      }
 
     case STRIP_COMPONENTS_OPTION:
       {
@@ -1969,7 +2090,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case WARNING_OPTION:
       set_warning_option (arg);
       break;
-      
+
     case '0':
     case '1':
     case '2':
@@ -2034,28 +2155,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
 #endif /* not DEVICE_PREFIX */
 
-    case '?':
-      tar_help (state);
-      close_stdout ();
-      exit (0);
-
-    case USAGE_OPTION:
-      argp_state_help (state, state->out_stream, ARGP_HELP_USAGE);
-      close_stdout ();
-      exit (0);
-
-    case VERSION_OPTION:
-      version_etc (state->out_stream, "tar", PACKAGE_NAME, VERSION,
-		   "John Gilmore", "Jay Fenlason", (char *) NULL);
-      close_stdout ();
-      exit (0);
-
-    case HANG_OPTION:
-      _argp_hang = atoi (arg ? arg : "3600");
-      while (_argp_hang-- > 0)
-	sleep (1);
-      break;
-
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -2068,7 +2167,7 @@ static struct argp argp = {
   N_("[FILE]..."),
   doc,
   NULL,
-  NULL,
+  tar_help_filter,
   NULL
 };
 
@@ -2096,12 +2195,20 @@ find_argp_option (struct argp_option *o, int letter)
   return NULL;
 }
 
+static const char *tar_authors[] = {
+  "John Gilmore",
+  "Jay Fenlason",
+  NULL
+};
+
 static void
 decode_options (int argc, char **argv)
 {
   int idx;
   struct tar_args args;
 
+  argp_version_setup ("tar", tar_authors);
+
   /* Set some default option values.  */
   args.textual_date = NULL;
   args.wildcards = default_wildcards;
@@ -2113,7 +2220,7 @@ decode_options (int argc, char **argv)
   args.version_control_string = 0;
   args.input_files = false;
   args.compress_autodetect = false;
-  
+
   subcommand_option = UNKNOWN_SUBCOMMAND;
   archive_format = DEFAULT_FORMAT;
   blocking_factor = DEFAULT_BLOCKING;
@@ -2132,7 +2239,9 @@ decode_options (int argc, char **argv)
   check_device_option = true;
 
   incremental_level = -1;
-  
+
+  seek_option = -1;
+
   /* Convert old-style tar call by exploding option element and rearranging
      options accordingly.  */
 
@@ -2194,8 +2303,7 @@ decode_options (int argc, char **argv)
 
   prepend_default_options (getenv ("TAR_OPTIONS"), &argc, &argv);
 
-  if (argp_parse (&argp, argc, argv, ARGP_IN_ORDER|ARGP_NO_HELP,
-		  &idx, &args))
+  if (argp_parse (&argp, argc, argv, ARGP_IN_ORDER, &idx, &args))
     exit (TAREXIT_FAILURE);
 
 
@@ -2264,18 +2372,6 @@ decode_options (int argc, char **argv)
 			  _("--occurrence cannot be used in the requested operation mode")));
     }
 
-  if (seekable_archive && subcommand_option == DELETE_SUBCOMMAND)
-    {
-      /* The current code in delete.c is based on the assumption that
-	 skip_member() reads all data from the archive. So, we should
-	 make sure it won't use seeks. On the other hand, the same code
-	 depends on the ability to backspace a record in the archive,
-	 so setting seekable_archive to false is technically incorrect.
-         However, it is tested only in skip_member(), so it's not a
-	 problem. */
-      seekable_archive = false;
-    }
-
   if (archive_names == 0)
     {
       /* If no archive file name given, try TAPE from the environment, or
@@ -2300,7 +2396,7 @@ decode_options (int argc, char **argv)
   if (incremental_level != -1 && !listed_incremental_option)
     WARN ((0, 0,
 	   _("--level is meaningless without --listed-incremental")));
-  
+
   if (volume_label_option)
     {
       if (archive_format == GNU_FORMAT || archive_format == OLDGNU_FORMAT)
@@ -2362,7 +2458,7 @@ decode_options (int argc, char **argv)
     old_files_option = UNLINK_FIRST_OLD_FILES;
 
 
-  if (test_label_option)
+  if (subcommand_option == TEST_LABEL_SUBCOMMAND)
     {
       /* --test-label is silent if the user has specified the label name to
 	 compare against. */
@@ -2378,7 +2474,7 @@ decode_options (int argc, char **argv)
   if (same_order_option && listed_incremental_option)
     USAGE_ERROR ((0, 0, _("--preserve-order is not compatible with "
 			  "--listed-incremental")));
-  
+
   /* Forbid using -c with no input files whatsoever.  Check that `-f -',
      explicit or implied, is used correctly.  */
 
@@ -2397,6 +2493,7 @@ decode_options (int argc, char **argv)
     case EXTRACT_SUBCOMMAND:
     case LIST_SUBCOMMAND:
     case DIFF_SUBCOMMAND:
+    case TEST_LABEL_SUBCOMMAND:
       for (archive_name_cursor = archive_name_array;
 	   archive_name_cursor < archive_name_array + archive_names;
 	   archive_name_cursor++)
@@ -2445,9 +2542,8 @@ decode_options (int argc, char **argv)
     }
 
   checkpoint_finish_compile ();
-  
-  if (verbose_option)
-    report_textual_dates (&args);
+
+  report_textual_dates (&args);
 }
 
 
@@ -2458,7 +2554,7 @@ int
 main (int argc, char **argv)
 {
   set_start_time ();
-  program_name = argv[0];
+  set_program_name (argv[0]);
 
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
@@ -2481,14 +2577,12 @@ main (int argc, char **argv)
 
   obstack_init (&argv_stk);
 
-  /* Ensure default behavior for some signals */
-  signal (SIGPIPE, SIG_DFL);
   /* System V fork+wait does not work if SIGCHLD is ignored.  */
   signal (SIGCHLD, SIG_DFL);
 
   /* Try to disable the ability to unlink a directory.  */
   priv_set_remove_linkdir ();
-  
+
   /* Decode options.  */
 
   decode_options (argc, argv);
@@ -2504,7 +2598,7 @@ main (int argc, char **argv)
     {
     case UNKNOWN_SUBCOMMAND:
       USAGE_ERROR ((0, 0,
-		    _("You must specify one of the `-Acdtrux' options")));
+		    _("You must specify one of the `-Acdtrux' or `--test-label'  options")));
 
     case CAT_SUBCOMMAND:
     case UPDATE_SUBCOMMAND:
@@ -2538,6 +2632,9 @@ main (int argc, char **argv)
       diff_init ();
       read_and (diff_archive);
       break;
+
+    case TEST_LABEL_SUBCOMMAND:
+      test_archive_label ();
     }
 
   if (totals_option)
@@ -2560,7 +2657,7 @@ main (int argc, char **argv)
   if (stdlis == stdout)
     close_stdout ();
   else if (ferror (stderr) || fclose (stderr) != 0)
-    exit_status = TAREXIT_FAILURE;
+    set_exit_status (TAREXIT_FAILURE);
 
   return exit_status;
 }
@@ -2598,3 +2695,13 @@ tar_timespec_cmp (struct timespec a, struct timespec b)
     a.tv_nsec = b.tv_nsec = 0;
   return timespec_cmp (a, b);
 }
+
+/* Set tar exit status to VAL, unless it is already indicating
+   a more serious condition. This relies on the fact that the
+   values of TAREXIT_ constants are ranged by severity. */
+void
+set_exit_status (int val)
+{
+  if (val > exit_status)
+    exit_status = val;
+}
